Ext.namespace('dSS');

dSS.ZoneView = Ext.extend(Ext.DataView, {
    constructor: function(config) {

        this.featureConfig = dSS.config.FeatureConfig.ZONE;
        this.lastSelectedZoneID = undefined;

        if (config.featureConfig != undefined) {
            this.featureConfig = config.featureConfig;
        }
        var zoneTemplate = new Ext.XTemplate(
            '<tpl for=".">', 
                '<div class="zone-wrap {css}" id="zone-{id}">', 
                    '<span>', 
                        '{name}',
                        '<tpl if="(unnamedDevices === true)">',
                            ' *',
                        '</tpl>',
                    '</span>', 
                '</div>', 
            '</tpl>', 
            '<div class="x-clear"></div>');

        Ext.apply(this, {
            store: dSS.data.Loader.getZoneStore(),
            tpl: zoneTemplate,
            singleSelect: true,
            multiSelect: false,
            layout: 'fit',
            style: "overflow: auto",
            overClass: 'x-view-over',
            itemSelector: 'div.zone-wrap',
            emptyText: dSS._("No rooms to display")
        });

        dSS.ZoneView.superclass.constructor.apply(this, arguments);

        // Here you can add functionality that requires the object to
        // exist, like event handling.
        this.on('selectionchange', function(dv, nodes) {
            if (nodes.length > 0) {
                var selectedZones = this.getSelectedRecords();
                if (selectedZones.length > 0) {
                    this.lastSelectedZoneID = selectedZones[0].get('id');
                }
            }

            this.filterDevices();
        },
        this);

        dSS.MessageBus.on('structureloaded', function() {
            // #7453 can't say why "this.isVisible()" is not working properly,
            // same goes for the first owner container, so we have to go two
            // levels up to reach the panel that is actually the "tab" itself,
            // in this case it will be either the dsszonebrowser or the
            // dsssensorbrowser
            if (this.ownerCt.ownerCt) {
                if (!this.ownerCt.ownerCt.isVisible()) {
                    return;
                }
            }

            if (!this.ownerCt.isVisible()) {
                return;
            }

            if (!this.isVisible()) {
                return;
            }

            if (this.getStore().getCount() === 0) {
                this.getStore().clearFilter();
                this.filterDevices();
                return;
            }

            this.restoreSelection();
        },
        this);

        this.on('render', function() {
            var zoneView = this;
            this.dropZone = new Ext.dd.DropZone(zoneView.getEl(), {
                ddGroup: "zoneDeviceDD",
                containerScroll: true,
                getTargetFromEvent: function(e) {
                    return e.getTarget(zoneView.itemSelector);
                },
                onNodeEnter: function(target, dd, e, data) {
                    Ext.fly(target).addClass('my-row-highlight-class');
                },
                onNodeOut: function(target, dd, e, data) {
                    Ext.fly(target).removeClass('my-row-highlight-class');
                },
                onNodeOver: function(target, dd, e, data) {
                    var record = zoneView.getRecord(target);
                    if (record.get('id') === 0) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }
                    return Ext.dd.DropZone.prototype.dropAllowed;
                },
                onNodeDrop: function(target, dd, e, data) {
                    var record = zoneView.getRecord(target);
                    if (record.get('id') === 0) {
                        return false;
                    }

                    Ext.each(data.selections, function(device) {
                        var currentDevice = device;

                        if (currentDevice.get('isPresent') === false) {
                            return;
                        }

                        if (currentDevice.get('zoneId') === record.get('id')) {
                            dSS.MessageBus.fireEvent('statusmessage',
                                dSS.gt.strargs(
                                    dSS._("Device %1 could not be moved into " +
                                          "zone %2: device is already in this" +
                                          " zone"),
                                    [
                                        '&quot;' +
                                        dSS.util.getName(
                                            currentDevice.get('name')) +
                                        '&quot; (' +
                                         currentDevice.get('DisplayID') +
                                         ')',
                                         '&quot;' + record.get('name') +
                                         '&quot;'
                                    ]
                                )
                            );
                            return;
                        }

                        Ext.Ajax.request({
                            url: '/json/structure/zoneAddDevice',
                            disableCaching: true,
                            method: "GET",
                            scope: zoneView,
                            params: {
                                dsuid: currentDevice.get('dsuid'),
                                zone: record.get('id')
                            },
                            success: function(result, request) {
                                try {
                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                    if (jsonData.ok) {
                                        this.moveDevices(jsonData.result,
                                            record,
                                            currentDevice.get('zoneId'));
                                    } else {
                                        Ext.Msg.alert(dSS._("Error"),
                                            dSS._("Device") +
                                            ' ' + currentDevice.get('DisplayID') +
                                            ' ' + dSS._("could not be moved") +
                                            ': (' + jsonData.message + ')');
                                    }
                                    dSS.data.Loader.getDeviceStore().getSensorAssignments();
                                } catch(err) {
                                    Ext.Msg.alert(
                                        dSS._("Error"), dSS._("Device") + ' ' +
                                        currentDevice.get('DisplayID') + ' ' +
                                        dSS._("could not be moved") +
                                        ': (' + err + ')');
                                    dSS.MessageBus.fireEvent('reloadapartment');
                                }
                            },
                            failure: function(result, request) {
                                Ext.Msg.alert(
                                    dSS._("Error"), dSS._("Device") +  ' ' +
                                    currentDevice.get('DisplayID') + ' ' +
                                    dSS._("could not be moved"));
                                dSS.MessageBus.fireEvent('reloadapartment');
                            }
                        });
                    });
                    return true;
                }
            });
        },
        this);

        this.on('contextmenu', function(view, index, node, event) {
            event.preventDefault();
            event.stopEvent();

            if (this.featureConfig !== dSS.config.FeatureConfig.ZONE) {
                return;
            }

            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }
            this.select(index);

            var menuItem;

            menuItem = new Ext.menu.Item({
                text: dSS._("Blink"),
                iconCls: 'blink',
                handler: this.blinkZone.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Create new room"),
                iconCls: 'addZoneIcon',
                handler: this.createNewZone.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Rename room"),
                iconCls: 'renameZoneIcon',
                handler: this.renameZone.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Delete room"),
                iconCls: 'removeZoneIcon',
                handler: this.removeZone.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);


            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },

    moveDevices: function(json, newZone, oldZoneId) {
        if (json.movedDevices && json.movedDevices.length > 0) {
            var store = dSS.data.Loader.getDeviceStore();
            for (var d = 0; d < json.movedDevices.length; d++) {
                var i = store.findExact('dsuid', json.movedDevices[d].dSUID);
                if (i < 0) {
                    continue;
                }
                var device = store.getAt(i);
                device.set('zoneId', newZone.get('id'));
                device.set('zoneName', newZone.get('name'));
                device.set('assignedSensorTypes', []);
                device.commit();
                dSS.data.Loader.getZoneStore().fireEvent('zonedevnamechanged',
                                                         device);
                dSS.MessageBus.fireEvent('statusmessage',
                   dSS.gt.strargs(
                        dSS._("Device %1 was moved " +
                             "into zone %2"),
                        [
                            '&quot;' + dSS.util.getName(device.get('name')) +
                            '&quot; (' + device.get('DisplayID') +
                            ')',
                            '&quot;' + newZone.get('name') + '&quot;'
                        ]
                    )
                );
            }

            this.filterDevices();
            dSS.MessageBus.fireEvent('scenesmarkdirty');
            dSS.data.Loader.getZoneStore().fireEvent('zonedeviceleft',
                                                     oldZoneId);
        }
    },

    restoreSelection: function() {
        // no action if there is a valid selection
        var selectedZones = this.getSelectedRecords();
        if (selectedZones.length > 0) {
            this.filterDevices();
            return;
        }

        if (this.lastSelectedZoneID !== undefined) {
            var rec = this.getStore().getById(this.lastSelectedZoneID);
            if (rec !== undefined) {
                this.select(rec);
            } else {
                this.select(this.store.getById(0));
            }
        } else {
            this.select(this.store.getById(0));
        }
    },
    filterDevices: function() {
        var selectedZone;
        var selectedZones = this.getSelectedRecords(); // single select
        if (selectedZones.length > 0) {
            selectedZone = selectedZones[0];
        }
        if (this.featureConfig === dSS.config.FeatureConfig.ZONE) {
            dSS.MessageBus.fireEvent('filter_devices_zone', selectedZone);
        } else if (this.featureConfig === dSS.config.FeatureConfig.SENSOR) {
            dSS.MessageBus.fireEvent('filter_devices_sensor', selectedZone);
        }
    },

    removeZone: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No room selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        var deviceStore = dSS.data.Loader.getDeviceStore();

        if (deviceStore.getRange().length > 0) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Can not delete a non empty room"));
            return;
        }

        Ext.Ajax.request({
            url: '/json/structure/removeZone',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                zoneID: record.get('id')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        this.getStore().remove(record);
                        dSS.MessageBus.fireEvent('statusmessage',
                          dSS._("Zone") + ' &quot;' +
                             record.get('name') +
                            '&quot; ' + dSS._("was removed"));
                    } else {
                        Ext.Msg.alert(dSS._("Error"),
                                     dSS._("Could not remove room") + ': ' +
                                     jsonData.message);
                    }
                } catch(error) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not remove room") + ': ' +
                                  error);
                }
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not remove room"));
            }
        });
    },

    blinkZone: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No room selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        Ext.Ajax.request({
            url: '/json/zone/blink',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                id: record.get('id')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (!jsonData.ok) {
                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not blink room") + " \"" + record.get('name') + "\"");
                    } else {
                        dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Zone") + ' &quot;' + record.get('name') +
                          '&quot; ' + dSS._("is blinking now"));
                    }
                } catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not blink room") + " \"" + record.get('name') + "\"");
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not blink room") + " \"" + record.get('name') + "\"");
            }
        });
    },
    renameZone: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No room selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        if (record.get('id') === 0) {
            Ext.Msg.alert(dSS._("Error"),
                          dSS._("Renaming 'All devices' entry is not allowed"));
            return;
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: this,
            title: dSS._("Rename room"),
            jumpMode: true,
            featureConfig: dSS.config.FeatureConfig.ZONE,
            callerScope: this,
            width: 350,
            dialogType: dSS.config.dialogTypes.RENAME_DIALOG,
            maskElement: this.findParentByType(Ext.Viewport).el
        });
        win.show();
    },
    createNewZone: function() {
        var zoneStore = this.getStore();
        Ext.Msg.prompt(dSS._("Create new room"), dSS._("Name for new room"), function(btn, text) {
            if (btn === 'ok') {
                var i;
                for (i = 0; i <= zoneStore.data.length; i++) {
                    if (zoneStore.findExact('id', i) === - 1) {

                        Ext.Ajax.request({
                            url: '/json/structure/addZone',
                            disableCaching: true,
                            method: "GET",
                            params: {
                                zoneID: i
                            },
                            success: function(result, request) {
                                try {
                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                    if (jsonData.ok) {
                                       dSS.MessageBus.fireEvent('statusmessage',
                                       dSS._("Zone") + ' #' + i +
                                        ' ' + dSS._("was created"));

                                        Ext.Ajax.request({
                                            url: '/json/zone/setName',
                                            disableCaching: true,
                                            method: "GET",
                                            params: {
                                                id: i,
                                                newName: text
                                            },
                                            success: function(result, request) {
                                                try {
                                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                                    if ((text === undefined) ||
                                                        (text.length === 0)) {
                                                        text = dSS._("Zone") +
                                                               ' #' + i;
                                                    }
                                                    if (jsonData.ok) {
                                                        zoneStore.loadData({
                                                            zones: [
                                                            {
                                                                id: i,
                                                                name: text
                                                            }]
                                                        },
                                                        true);
                                                    } else {
                                                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone") + ": " + jsonData.message);
                                                    }
                                                }
                                                catch(err) {
                                                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone") + ": " + err);
                                                }
                                            },
                                            failure: function(result, request) {
                                                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone"));
                                            }
                                        });

                                    }
                                }
                                catch(err) {
                                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone"));
                                }
                            },
                            failure: function(result, request) {
                                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone"));
                            }
                        });
                        return;
                    }
                }
            }
        });
    }
});

Ext.reg('dsszoneview', dSS.ZoneView);

