//= require <external/IconCombo>

Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID, group (configured joker group)
dSS.devprops.JokerConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dSUID === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSUID"));
            return;
        }

        this.userGroupWarned = false;

        var list =
        [
            [
                dSS.config.groups.LIGHT,
                dSS.config.GroupColorIconCls[dSS.config.groups.LIGHT],
                dSS._("Light")
            ],
            [
                dSS.config.groups.SHADOW,
                dSS.config.GroupColorIconCls[dSS.config.groups.SHADOW],
                dSS._("Shadow")
            ],
            [
                dSS.config.groups.HEATING,
                dSS.config.GroupColorIconCls[dSS.config.groups.HEATING],
                dSS._("Heating")
            ],
            [
                dSS.config.groups.AUDIO,
                dSS.config.GroupColorIconCls[dSS.config.groups.AUDIO],
                dSS._("Audio")
            ],
            [
                dSS.config.groups.VIDEO,
                dSS.config.GroupColorIconCls[dSS.config.groups.VIDEO],
                dSS._("Video")
            ],
            [
                dSS.config.groups.JOKER,
                dSS.config.GroupColorIconCls[dSS.config.groups.JOKER],
                dSS._("Joker")
            ]
        ];


        // #1891 - keep value but replace with "reserved" entry
        if ((config.group === dSS.config.groups.ACCESS) ||
            (config.group === dSS.config.groups.SECURITY)) {
            list.push(
                [
                    config.group,
                    '', // no icon
                    dSS._("Reserved")
                ]);
        }

        this.dSUID = config.dSUID;
        this.device = config.device;
        this.group = config.group;

        this.addEvents('formsubmitted', 'formloaded');

        this.colorSelection = new Ext.ux.IconCombo({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 145,
            mode: 'local',
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            fieldLabel: dSS._("Color group"),
            tpl: new Ext.XTemplate(
                '<tpl for="."><div class="x-combo-list-item">',
                '<table><tbody><tr>',
                '<td>',
                '<div class="{css} x-icon-combo-icon"></div></td>',
                '<td>{displayText}</td>',
                '</tr></tbody></table>',
                '</div></tpl>'),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'css',
                    'displayText'
                ],
                data: list
            }),
            valueField: 'id',
            displayField: 'displayText',
            iconClsField: 'css'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.colorSelection
                    ]
                }
            ]
        });

        this.firstLoad = true;

        dSS.devprops.JokerConfigForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.colorSelection.on('select', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('afterrender', function() {
                this.fireEvent('formloaded', true);
            },
            this);

        this.colorSelection.on('afterrender', function() {
            this.colorSelection.on('select', function(box, record, index) {
                    dSS.MessageBus.fireEvent('jokerconfigformchanged',
                                              record.data.id);
                },
                this);

                if ((this.closed !== true) &&
                   ((this.group >= dSS.config.groups.LIGHT) &&
                    (this.group <= dSS.config.groups.JOKER))) {
                    this.colorSelection.setValue(this.group);
                }
            },
            this);

        this.colorSelection.on('expand', function(box, record, index) {
                var devGroups = this.device.get('groups');
                var name = this.device.get('name');
                var i;
                if (this.userGroupWarned === true) {
                    return true;
                }
                for (i = 0; i < devGroups.length; i++) {
                    if (dSS.util.isUserGroup(devGroups[i])) {
                        this.userGroupWarned = true;
                        box.collapse();
                        Ext.Msg.alert(dSS._("Warning"),
                            dSS.gt.strargs(
                                dSS._("Device %1 %2 is member in one or more " +
                                      "user groups. Changing device color " +
                                      "configuration will remove the device " +
                                      "from all user groups."),
                                [
                                    '&quot;' + name + '&quot;',
                                    '(' + this.device.get('DisplayID') + ')'
                                ]
                            )
                        );
                        return false;
                    }
                }
                return true;
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('submit', function() {
                var value = this.colorSelection.getValue();
                var dSUID = this.dSUID;
                var group = this.group;
                if (value == group) {
                    this.fireEvent('formsubmitted');
                    return;
                }

                Ext.Ajax.request({
                    url: '/json/device/setJokerGroup',
                    disableCaching: true,
                    method: "GET",
                    params:
                    {
                        dsuid : this.dSUID,
                        groupID : value
                    },
                    scope: this,
                    success: function(result, request) {
                        try {
                            var json = Ext.util.JSON.decode(result.responseText);
                            if (!json.ok) {
                                Ext.Msg.alert(dSS._("Error"),
                                    dSS._("Could not set joker device group") +
                                    ': (' + json.message + ')');
                                this.fireEvent('formsubmitted');
                                return;
                            }
                            var devStore = dSS.data.Loader.getDeviceStore();
                            if (json.result.action === "update") {
                                var i;
                                var zoneStore = dSS.data.Loader.getZoneStore();
                                for (i = 0; i < json.result.devices.length; i++) {
                                    var zone = zoneStore.getById(json.result.devices[i].zoneID);
                                    if (zone !== undefined) {
                                        json.result.devices[i].zoneName = zone.get('name');
                                    }
                                    json.result.devices[i].firstSeen =
                                        Date.parseDate(
                                           json.result.devices[i].firstSeen,
                                           "Y-m-d H:i:s", true);
                                    json.result.devices[i].inactiveSince =
                                        Date.parseDate(
                                            json.result.devices[i].inactiveSince,
                                            "Y-m-d H:i:s", true);
                                }

                                // parser is set to devices root element
                                devStore.loadData(json.result, true);
                            }
                            dSS.MessageBus.fireEvent('scenesmarkdirty');
                        } catch (err) {
                            Ext.Msg.alert(dSS._("Error"),
                                dSS._("Could not set joker device group") +
                                ': (' + err + ')');
                        }
                        this.fireEvent('formsubmitted');
                    },
                    failure: function(result, request) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not set joker device group"));
                        this.fireEvent('formsubmitted');
                    }
                });
        },
        this);
    }
});

Ext.reg('dssdevjokerconfigform', dSS.devprops.JokerConfigForm);

